﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

// Added using statements.
using System.Data.Objects;
using System.Threading;

namespace TestModelFirst
{
   public partial class Form1 : Form
   {
      public Form1()
      {
         InitializeComponent();
      }

      private void btnAdd_Click(object sender, EventArgs e)
      {
         // Create a new purchase.
         Purchases NewPurchase = new Purchases();
         NewPurchase.Amount = new Decimal(5.99);
         NewPurchase.PurchaseDate = DateTime.Now;

         // Create a new customer and add the purchase.
         Customers NewCustomer = new Customers();
         NewCustomer.CustomerName = "Josh Bailey";

         // Create the context.
         Rewards2ModelContainer context = new Rewards2ModelContainer();

         // Add the record and save it.
         context.Customers.Add(NewCustomer);
         context.Purchases.Add(NewPurchase);
         context.SaveChanges();

         // Display a success message.
         MessageBox.Show("Record Added");
      }

      private void btnQuery_Click(object sender, EventArgs e)
      {
          // Create the context.
          Rewards2ModelContainer context = new Rewards2ModelContainer();

          // Obtain the customer list.
          var CustomerList =
              from cust in context.Customers
              select cust;

          //// Obtain the customer list without change tracking.
          //var CustomerList =
          //    from cust in context.Customers.AsNoTracking()
          //    select cust;

          //// Obtain the customer list in list form.
          //List<Customers> CustomerList =
          //    (from cust in context.Customers
          //     select cust).ToList<Customers>();

          // Process each customer in the list.
          StringBuilder Output = 
              new StringBuilder("Customer List:");
          foreach (var Customer in CustomerList)
          {
              // Create a customer entry for each customer.
              Output.Append("\r\n" + Customer.CustomerName + 
                  " has made purchases on: ");

              // Process each purchase for that particular customer.
              foreach (var Purchase in Customer.Purchases)
                  Output.Append("\r\n\t" + Purchase.PurchaseDate);
          }

          // Display the result on screen.
          MessageBox.Show(Output.ToString());
      }

      // This delegate defines the method used to retrieve data from the thread.
      public delegate void ContextCallback(IQueryable<Customers> CustomerList);

      private void btnThreaded_Click(object sender, EventArgs e)
      {
         // Create an instance of the thread class.
         MultiThreadedQuery MTQ = new MultiThreadedQuery(
            new ContextCallback(ResultsCallback));

         // Create a thread to perform the task.
         Thread MyThread = new Thread(MTQ.GetCustomers);

         // Start the thread.
         MyThread.Start();
      }

      public static void ResultsCallback(IQueryable<Customers> CustomerList)
      {
         // Process each customer in the list.
         StringBuilder Output =
             new StringBuilder("Customer List:");
         foreach (var Customer in CustomerList)
         {
            // Create a customer entry for each customer.
            Output.Append("\r\n" + Customer.CustomerName +
                " has made purchases on: ");

            // Process each purchase for that particular customer.
            foreach (var Purchase in Customer.Purchases)
               Output.Append("\r\n\t" + Purchase.PurchaseDate);
         }

         // Display the result on screen.
         MessageBox.Show(Output.ToString());
      }
   }

   public class MultiThreadedQuery
   {
      // The context used to access the data source.
      private Rewards2ModelContainer context;

      // A list of customers retrieved from the database.
      private IQueryable<Customers> CustomerList;

      // This delegate is used to call the callback method when the task is
      // complete.
      private readonly Form1.ContextCallback _Callback;

      public MultiThreadedQuery(Form1.ContextCallback CallbackDelegate)
      {
         // Assign a method to the callback delegate.
         _Callback = CallbackDelegate;
      }

      public void GetCustomers()
      {
         // Make sure the context points to the data source.
         if (context == null)
            context = new Rewards2ModelContainer();

         // You must lock this section of code to ensure it's thread safe.
         lock (context)
         {
            // Obtain the customer list.
            CustomerList =
                from cust in context.Customers
                select cust;

            // Return the result to the caller, when there is a caller.
            if (_Callback != null)
               _Callback(CustomerList);
         }
      }
   }
}
